import uuid
from urllib.parse import urlencode, quote, urlparse, parse_qs
from weishijiajie.rsa_util import RSAUtil  # 假设你之前已转换好的
from weishijiajie.param_entity import ParamEntity  # 假设你之前已转换好的


class SignUtilsMOP:
    @staticmethod
    def do_sign(servlet_path: str, private_key: str, param_entity: ParamEntity) -> str:
        if not servlet_path.strip():
            raise Exception("url path is empty!")
        if not private_key.strip():
            raise Exception("privateKey is empty!")

        public_map = {
            "method": param_entity.method,
            "status": param_entity.status,
            "format": param_entity.format,
            "flowdId": str(uuid.uuid4()).replace("-", "")
        }

        if param_entity.query_map:
            public_map.update(param_entity.query_map)

        try:
            sign = RSAUtil.sign(public_map, private_key)
            public_map["sign"] = quote(sign, encoding="utf-8")

            query_string = "&".join(f"{k}={v}" for k, v in public_map.items())
            return f"{servlet_path}?{query_string}"
        except Exception as e:
            print("sign failed!")
            raise e

    @staticmethod
    def verify(sign_url: str, public_key: str) -> bool:
        if "?" not in sign_url:
            return False

        param_str = sign_url.split("?", 1)[1]
        param_map = {}

        for pair in param_str.split("&"):
            if pair.strip():
                k, v = pair.split("=", 1)
                param_map[k] = v

        try:
            return RSAUtil.verify(param_map, public_key)
        except Exception:
            return False

